/*******Test ranks of 100,000 6x8 binary matrices**************
 *******Each row a byte from a RNG, overlapping rows*************/

#include "header.h"
#include "macro.h"

/* define a binary matrix */
typedef struct binmatrix
{

  uniform *row;
  int no_row, no_col;
  unsigned long mask;

} binmatrix;

/*print the title*/
void
rnk_ttl (char *fn, char *test)
{

  printf ("\t\tRANK TEST for binary matrices (%s) from %s\n", test, fn);

  return;
}

/*compute the rank of a binary matrix*/
int
rkbm (binmatrix bm)
{
  register int i, j, k, rt = 0;
  int rank = 0;
  uniform tmp;

  for (k = 0; k < bm.no_row; ++k)
    {
      i = k;

      while (((bm.row[i] >> rt) & 1) == 0)
	{
	  ++i;

	  if (i < bm.no_row)
	    {
	      continue;
	    }
	  else
	    {
	      ++rt;
	      if (rt < bm.no_col)
		{
		  i = k;
		  continue;
		}
	    }

	  return rank;
	}

      ++rank;
      if (i != k)
	{
	  tmp = bm.row[i];
	  bm.row[i] = bm.row[k];
	  bm.row[k] = tmp;
	}

      for (j = i + 1; j < bm.no_row; ++j)
	{
	  if (((bm.row[j] >> rt) & 1) == 0)
	    continue;
	  else
	    bm.row[j] ^= bm.row[k];
	}

      ++rt;
    }

  return rank;
}

/* perform the test and calculate test-stat */
real
rnk_stat (char *fn, counter no_mtr, char *test, int rt)
{
  real p6[] = { .009443, 0.217439, 0.773118 };
  real p30[] = { .0052854502, .1283502644, .5775761902, .2887880952 };

  binmatrix bm;
  register counter i, j;
  register int cls, llim;
  char cat[2][4] = { {"r<="}, {"r="} };
  counter *f;

  short df = 3;
  real pvalue, *p;
  real Ef, chsq = 0, tmp;

  switch (test[1])
    {
    case '1':
      bm.no_row = bm.no_col = 31;
      bm.mask = pow (2, 31) - 1;
      llim = 28;
      p = p30;
      break;
    case '2':
      bm.no_row = bm.no_col = 32;
      bm.mask = pow (2, 32) - 1;
      llim = 29;
      p = p30;
      break;
    default:
      {
	df = 2;
	bm.no_row = 6;
	bm.no_col = 8;
	bm.mask = pow (2, 8) - 1;
	llim = 4;
	p = p6;
	//    printf("\n\t\t\t      bits %2d to %2d\n", 25-rt, 32-rt);
	break;
      }
    }

  bm.row = (uniform *) malloc (bm.no_row * sizeof (uniform));

  f = (counter *) calloc ((df + 1), sizeof (counter));


  for (i = 1; i <= no_mtr; ++i)
    {

      /* get the rows of a matrix */
      for (j = 0; j < bm.no_row; ++j)
	{
	  switch (test[1])
	    {
	    case '1':
	      bm.row[j] = (uni (fn) >> 1);
	      break;
	    case '2':
	      bm.row[j] = uni (fn);
	      break;
	    default:
	      bm.row[j] = (uni (fn) >> rt);
	      break;
	    }
	  bm.row[j] &= bm.mask;
	}

      cls = rkbm (bm);
      cls = MAX (llim, cls) - llim;
      ++f[cls];
    }

  /* compute chi-square */
  for (i = 0; i <= df; ++i)
    {
      Ef = no_mtr * (*(p + i));
      tmp = (f[i] - Ef) * (f[i] - Ef) / Ef;
      chsq += tmp;

    }
  uni ("close");

  /*  cfree(f, (df+1), sizeof(counter)); */
  free (bm.row);

  pvalue = 1 - Chisq (df, chsq);

  switch (test[1])
    {
    case '1':
      printf ("\n\t\tp-value = %.6f\n\n", pvalue);
      results.rank31 = pvalue;
      break;
    case '2':
      printf ("\n\t\tp-value = %.6f\n\n", pvalue);
      results.rank32 = pvalue;
      break;
    default:
      break;
    }



  return pvalue;
}

/* type "6x8", "31x31" or "32x32" (including the quotation mark) to call each test.*/
void
binrnk (char *filename, char *test)
{
  counter no_matrices = 100000;
  short rt = 24, not6x8 = strncmp (test, "6x8", 1);
  real *p;

  if (not6x8)
    {
      rt = 0;
      no_matrices = 40000;
    }

  rnk_ttl (filename, test);

  p = (real *) malloc ((rt + 1) * sizeof (real));

  do
    {
      p[rt] = rnk_stat (filename, no_matrices, test, rt);
      --rt;
    }
  while (rt >= 0);

  if (not6x8)
    {
      printf
	("\t_____________________________________________________________\n\n");
      return;
    }

  printf ("\t\tp-value = %.6f\n\n", KStest (p, 25));
  results.rank6x8 = KStest (p, 25);
  printf
    ("\t_____________________________________________________________\n\n");

  free (p);

  return;
}
