/*
ENT  --  Entropy calculation and analysis of putative
random sequences.

Designed and implemented by John "Random" Walker in May 1985.

Multiple analyses of random sequences added in December 1985.

Bit stream analysis added in September 1997.

Terse mode output, getopt() command line processing,
optional stdin input, and HTML documentation added in
October 1998.

Documentation for the -t (terse output) option added
in July 2006.

Replaced table look-up for chi square to probability
conversion with algorithmic computation in January 2008.

For additional information and the latest version,
see http://www.fourmilab.ch/random/

Recoded by Alexander Myasnikov

Web: www.darksoftware.narod.ru

*/

#define _CRT_SECURE_NO_WARNINGS 1
#define _CRT_NONSTDC_NO_WARNINGS 1

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <fcntl.h>
#include <io.h>

#include "iso8859.h"
#include "randtest.h"

#include "header.h"

#define UPDATE  "March 03, 2009"

#define FALSE 0
#define TRUE  1

#define PI 3.14159265358979323846

extern double pochisq (const double ax, const int df);

/*  HELP  --  Print information on how to call	*/




#define STDOUT GetStdHandle(STD_OUTPUT_HANDLE)

void
gotoxy (int x, int y)
{
  COORD dwCursorPosition;
  dwCursorPosition.X = (SHORT) x;
  dwCursorPosition.Y = (SHORT) y;
  SetConsoleCursorPosition (STDOUT, dwCursorPosition);
}


void
clrscr (void)
{
  HANDLE hstdout = STDOUT;
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  if (GetConsoleScreenBufferInfo (hstdout, &csbi))
    {
      COORD coordScreen = { 0, 0 };
      DWORD cCharsWritten;
      DWORD dwConSize = csbi.dwSize.X * csbi.dwSize.Y;
      FillConsoleOutputCharacter (hstdout, ' ', dwConSize, coordScreen,
				  &cCharsWritten);
      FillConsoleOutputAttribute (hstdout, csbi.wAttributes, dwConSize,
				  coordScreen, &cCharsWritten);
      SetConsoleCursorPosition (hstdout, coordScreen);
    }
}




/*  Results record */

tresults results;


/*                  */



/*  Main program  */

int
main (int argc, char *argv[])
{
  long ccount[256];		/* Bins to count occurrences of values */
  long size, totalc = 0, i, j, cnt = 0;	/* Total character count */
  double montepi, chip, scc, ent, mean, chisq;
  unsigned char buf[262144];
  unsigned char ocb;
  char buffer[16384];
  char fn[16384];
  FILE *fp = NULL;

  clrscr ();

  if (argc < 2)
    {
      printf ("Need some file name!");
      return 2;
    }


  if ((fp = fopen (argv[1], "rb")) == NULL)
    {
      printf ("Cannot open file %s\n", argv[1]);
      return 2;
    }


  fseek (fp, 0, SEEK_END);
  size = ftell (fp);
  fseek (fp, 0, SEEK_SET);

  memset (ccount, 0, sizeof ccount);

  /* Initialise for calculations */

  rt_init (0);

  /* Scan input file and count character occurrences */

  while ((cnt = fread (&buf[0], 1, 262144, fp)) > 0)

    {

      for (i = 0; i != cnt; i++)
	{

	  ocb = buf[i];

	  totalc += 1;

	  {
	    ccount[ocb]++;	/* Update counter for this bin */
	  }
	  rt_add (&ocb, 1);
	}
      gotoxy (0, 0);
      printf ("Position %1.4f%%               ",
	      ((double) totalc / (double) size) * 100);

    }

  fclose (fp);

  /* Complete calculation and return sequence metrics */

  rt_end (&ent, &chisq, &mean, &montepi, &scc);


  /* Calculate probability of observed distribution occurring from
     the results of the Chi-Square test */

  chip = pochisq (chisq, (255));

  /* Print bin counts if requested */


  /* Print calculated results */


  {
    clrscr();
    j = sprintf (buffer, "ENT pseudorandom test battery.\n");
    j += sprintf (buffer + j, "Recoded edition by Alexander Myasnikov.\n");
    j += sprintf (buffer + j, "Web: www.darksoftware.narod.ru\n\n");
    j += sprintf (buffer + j, "Test results for file %s.\n", argv[1]);
    j += sprintf (buffer + j, "Entropy = %f bits per byte.\n", ent);
    j +=
      sprintf (buffer + j, "\nOptimum compression would reduce the size\n");
    j +=
      sprintf (buffer + j, "of this %ld bytes file by %d percent.\n\n",
	       totalc, (short) ((100 * ((8) - ent) / (8.0))));
    j +=
      sprintf (buffer + j,
	       "Chi square distribution for %ld samples is %1.2f (factor %1.7f), and randomly\n",
	       totalc, chisq, chisq / 256);
    if (chip < 0.0001)
      {
	j +=
	  sprintf (buffer + j,
		   "would exceed this value less than 0.01 percent of the times.\n\n");
      }
    else if (chip > 0.9999)
      {
	j +=
	  sprintf (buffer + j,
		   "would exceed this value more than than 99.99 percent of the times.\n\n");
      }
    else
      {
	j +=
	  sprintf (buffer + j,
		   "would exceed this value %1.2f percent of the times.\n\n",
		   chip * 100);
      }
    j +=
      sprintf (buffer + j,
	       "Arithmetic mean value of data bytes is %1.4f (%.1f = random).\n",
	       mean, 127.5);
    j +=
      sprintf (buffer + j,
	       "Monte Carlo value for Pi is %1.9f (error %1.2f percent).\n",
	       montepi, 100.0 * (fabs (PI - montepi) / PI));
    j += sprintf (buffer + j, "Serial correlation coefficient is ");
    if (scc >= -99999)
      {
	j +=
	  sprintf (buffer + j, "%1.6f (totally uncorrelated = 0.0).\n", scc);
      }
    else
      {
	j += sprintf (buffer + j, "undefined (all values equal!).\n");
      }
  }

  if (argc == 3)
    {
      fp = fopen (argv[2], "wt+");
    }
  else
    {

      strcpy (&fn[0], argv[1]);

      strcat (&fn[0], ".ent.txt");

      fp = fopen (&fn[0], "wt+");
    }


  if (size >= 20971520)
    {

      craptest (argv[1]);
      bday (argv[1]);
      osum (argv[1]);
      bitst (argv[1]);
      mindist (argv[1]);
      squeez (argv[1]);
      cnt1s (argv[1], "stream");
      binrnk (argv[1], "6x8");
      park (argv[1]);
      sphere (argv[1]);
      runtest (argv[1]);
      binrnk (argv[1], "31x31");
      binrnk (argv[1], "32x32");
      strcpy (&fn[0], argv[0]);
      strcat (&fn[0], ".dat");
      operm5 (argv[1], &fn[0]);


      j +=
	sprintf (buffer + j, "\n\nBIRTHDAY TEST p-value: %f\n\n",
		 results.birth);
      j +=
	sprintf (buffer + j, "BITSTREAM TEST p-value: %f\n\n",
		 results.bitstream);
      j +=
	sprintf (buffer + j, "CDPARK TEST p-value: %f\n\n", results.cdpark);
      j +=
	sprintf (buffer + j, "COUNT 1's TEST p-value: %f\n\n",
		 results.count1);
      j += sprintf (buffer + j, "CRAP TEST p-value: %f\n\n", results.crap);
      j +=
	sprintf (buffer + j, "MINDIST TEST p-value: %f\n\n", results.mindist);
      j += sprintf (buffer + j, "OSUM TEST p-value: %f\n\n", results.osum);
      j +=
	sprintf (buffer + j, "RANK 31x31 TEST p-value: %f\n\n",
		 results.rank31);
      j +=
	sprintf (buffer + j, "RANK 32x32 TEST p-value: %f\n\n",
		 results.rank32);
      j +=
	sprintf (buffer + j, "RANK 6x8 TEST p-value: %f\n\n",
		 results.rank6x8);
      j += sprintf (buffer + j, "RUNS TEST p-value: %f\n\n", results.runs);
      j +=
	sprintf (buffer + j, "SPHERE 3D TEST p-value: %f\n\n",
		 results.sphere3d);
      j +=
	sprintf (buffer + j, "SQUEEZE TEST p-value: %f\n\n", results.squeeze);
      j += sprintf (buffer + j, "PERM5 TEST p-value: %f\n\n", results.perm5);

    }
  else

    {

      j +=
	sprintf (buffer + j,
		 "\n\nPlease, give 20 or more Mb file to run some diehard tests.");

    }

  fputs (&buffer[0], fp);
  fclose (fp);



  return 0;
}
